@@ -46,7 +46,7 @@ BEGIN {
   
   ### how to display or parse the filename
   $QR1 = qr{\A(?:/[^/]+){2,}/(?:perl|lib)/(.+)\Z};
-  $QR2 = qr{\A.+?(\w+/[\w\.]+)\Z};
+  $QR2 = qr{\A.+?([\w\.\-]+/[\w\.\-]+)\Z};
 }
 
 ###----------------------------------------------------------------###
